@@ -27,8 +27,11 @@
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <netinet/in.h>
+#include <limits.h>
 
 #include "raw1394.h"
+#include "csr.h"
 #include "kernel-raw1394.h"
 #include "raw1394_private.h"
 
@@ -82,7 +85,7 @@ int _raw1394_sync_cb(struct raw1394_handle *unused, struct sync_cb_data *data,
 
 
 
-static unsigned int init_rawdevice(struct raw1394_handle *h)
+static unsigned int init_rawdevice(struct ieee1394_handle *h)
 {
         struct raw1394_request req;
 
@@ -101,7 +104,7 @@ static unsigned int init_rawdevice(struct raw1394_handle *h)
         }
 
         if (req.error) {
-                errno = 0;
+                errno = EPROTO;
                 return -1;
         }
         memset(h->buffer, 0, HBUF_SIZE);
@@ -110,91 +113,144 @@ static unsigned int init_rawdevice(struct raw1394_handle *h)
 }
 
 
-struct raw1394_handle *raw1394_new_handle(void)
+struct ieee1394_handle *ieee1394_new_handle(void)
 {
-        struct raw1394_handle *handle;
+        struct ieee1394_handle *handle;
+        const char *defaultDevice = "/dev/raw1394";
 
-        handle = malloc(sizeof(struct raw1394_handle));
+        handle = malloc(sizeof(struct ieee1394_handle));
         if (!handle) {
                 errno = ENOMEM;
                 return NULL;
         }
 
-        handle->fd = open("/dev/raw1394", O_RDWR);
+        handle->fd = open(getenv("RAW1394DEV") ? getenv("RAW1394DEV"): defaultDevice, O_RDWR);
         if (handle->fd < 0) {
-                free(handle);
-                return NULL;
+                /* failover to default in attempt to idiot proof */
+                handle->fd = open(defaultDevice, O_RDWR);
+                if (handle->fd < 0) {
+                        free(handle);
+                        return NULL;
+                }
         }
 
         handle->generation = init_rawdevice(handle);
         if (handle->generation == -1) {
+                /* failover to default in attempt to idiot proof */
                 close(handle->fd);
-                free(handle);
-                return NULL;
+                handle->fd = open(defaultDevice, O_RDWR);
+                if (handle->fd < 0) {
+                        free(handle);
+                        return NULL;
+                }
+                handle->generation = init_rawdevice(handle);
+                if (handle->generation == -1) {
+                        close(handle->fd);
+                        free(handle);
+                        return NULL;
+                }
         }
 
         handle->err = 0;
         handle->bus_reset_handler = bus_reset_default;
         handle->tag_handler = tag_handler_default;
         handle->arm_tag_handler = arm_tag_handler_default;
-        memset(handle->iso_handler, 0, sizeof(handle->iso_handler));
-	handle->iso_buffer = NULL;
-	handle->iso_mode = ISO_INACTIVE;
+        handle->iso_buffer = NULL;
+        handle->iso_mode = ISO_INACTIVE;
+        handle->iso_packet_infos = NULL;
         return handle;
 }
 
-void raw1394_destroy_handle(struct raw1394_handle *handle)
+void ieee1394_destroy_handle(struct ieee1394_handle *handle)
 {
 	if (handle) {
 		if(handle->iso_mode != ISO_INACTIVE) {
-			raw1394_iso_shutdown(handle);
+			ieee1394_iso_shutdown(handle);
 		}
 		close(handle->fd);
 		free(handle);
 	}
 }
 
-int raw1394_get_fd(struct raw1394_handle *handle)
+int ieee1394_get_fd(struct ieee1394_handle *handle)
 {
         return handle->fd;
 }
 
 unsigned int raw1394_get_generation(struct raw1394_handle *handle)
 {
-        return handle->generation;
+	if (!handle) {
+		errno = EINVAL;
+		return UINT_MAX;
+	}
+	if (handle->is_fw)
+		return handle->mode.fw->generation;
+	else
+		return handle->mode.ieee1394->generation;
 }
 
 void raw1394_update_generation(struct raw1394_handle *handle, unsigned int gen)
 {
-        handle->generation = gen;
+	if (!handle) {
+		return;
+	}
+	if (handle->is_fw)
+		handle->mode.fw->generation = gen;
+	else
+		handle->mode.ieee1394->generation = gen;
 }
 
-int raw1394_get_nodecount(struct raw1394_handle *handle)
+int ieee1394_get_nodecount(struct ieee1394_handle *handle)
 {
-        return handle->num_of_nodes;
+	if (!handle) {
+		errno = EINVAL;
+		return UINT_MAX;
+	}
+	return handle->num_of_nodes;
 }
 
-nodeid_t raw1394_get_local_id(struct raw1394_handle *handle)
+nodeid_t ieee1394_get_local_id(struct ieee1394_handle *handle)
 {
-        return handle->local_id;
+	if (!handle) {
+		errno = EINVAL;
+		return 0xFFFF;
+	}
+	return handle->local_id;
 }
 
-nodeid_t raw1394_get_irm_id(struct raw1394_handle *handle)
+nodeid_t ieee1394_get_irm_id(struct ieee1394_handle *handle)
 {
-        return handle->irm_id;
+	if (!handle) {
+		errno = EINVAL;
+		return 0xFFFF;
+	}
+	return handle->irm_id;
 }
 
 void raw1394_set_userdata(struct raw1394_handle *handle, void *data)
 {
-        handle->userdata = data;
+	if (!handle) {
+		return;
+	}
+	if (handle->is_fw)
+		handle->mode.fw->userdata = data;
+	else
+		handle->mode.ieee1394->userdata = data;
 }
 
 void *raw1394_get_userdata(struct raw1394_handle *handle)
 {
-        return handle->userdata;
+	if (!handle) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if (handle->is_fw)
+		return handle->mode.fw->userdata;
+	else
+		return handle->mode.ieee1394->userdata;
 }
 
-int raw1394_get_port_info(struct raw1394_handle *handle, 
+int ieee1394_get_port_info(struct ieee1394_handle *handle, 
                           struct raw1394_portinfo *pinf, int maxports)
 {
         int num;
@@ -230,7 +286,7 @@ int raw1394_get_port_info(struct raw1394_handle *handle,
 }
 
 
-int raw1394_set_port(struct raw1394_handle *handle, int port)
+int ieee1394_set_port(struct ieee1394_handle *handle, int port)
 {
         struct raw1394_request req;
 
@@ -268,23 +324,23 @@ int raw1394_set_port(struct raw1394_handle *handle, int port)
         }
 }
 
-raw1394handle_t raw1394_new_handle_on_port(int port)
+ieee1394handle_t ieee1394_new_handle_on_port(int port)
 {
-	raw1394handle_t handle = raw1394_new_handle();
+	ieee1394handle_t handle = ieee1394_new_handle();
 	if (!handle)
 		return NULL;
 
 tryagain:
-	if (raw1394_get_port_info(handle, NULL, 0) < 0) {
-		raw1394_destroy_handle(handle);
+	if (ieee1394_get_port_info(handle, NULL, 0) < 0) {
+		ieee1394_destroy_handle(handle);
 		return NULL;
 	}
 
-	if (raw1394_set_port(handle, port)) {
+	if (ieee1394_set_port(handle, port)) {
 		if (errno == ESTALE || errno == EINTR) {
 			goto tryagain;
 		} else {
-			raw1394_destroy_handle(handle);
+			ieee1394_destroy_handle(handle);
 			return NULL;
 		}
 	}
@@ -292,7 +348,7 @@ tryagain:
 	return handle;
 }
 
-int raw1394_reset_bus_new(struct raw1394_handle *handle, int type)
+int ieee1394_reset_bus_new(struct ieee1394_handle *handle, int type)
 {
         struct raw1394_request req;
 
@@ -313,7 +369,7 @@ int raw1394_reset_bus(struct raw1394_handle *handle)
         return raw1394_reset_bus_new (handle, RAW1394_LONG_RESET);
 }
 
-int raw1394_busreset_notify (struct raw1394_handle *handle, 
+int ieee1394_busreset_notify (struct ieee1394_handle *handle, 
                              int off_on_switch)
 {
         struct raw1394_request req;
@@ -329,7 +385,7 @@ int raw1394_busreset_notify (struct raw1394_handle *handle,
         return 0; /* success */
 }
 
-int raw1394_update_config_rom(raw1394handle_t handle, const quadlet_t
+int ieee1394_update_config_rom(ieee1394handle_t handle, const quadlet_t
         *new_rom, size_t size, unsigned char rom_version)
 {
         struct raw1394_request req;
@@ -348,7 +404,7 @@ int raw1394_update_config_rom(raw1394handle_t handle, const quadlet_t
         return status;
 }
 
-int raw1394_get_config_rom(raw1394handle_t handle, quadlet_t *buffer,
+int ieee1394_get_config_rom(ieee1394handle_t handle, quadlet_t *buffer,
         size_t buffersize, size_t *rom_size, unsigned char *rom_version)
 {
         struct raw1394_request req;
@@ -367,3 +423,110 @@ int raw1394_get_config_rom(raw1394handle_t handle, quadlet_t *buffer,
 
         return status;
 }
+
+int ieee1394_bandwidth_modify (raw1394handle_t handle, unsigned int bandwidth,
+	enum raw1394_modify_mode mode)
+{
+        quadlet_t buffer, compare, swap, new;
+        int retry = 3;
+        int result;
+        
+        if (bandwidth == 0)
+                return 0;
+        
+        /* Reading current bandwidth usage from IRM. */
+        result = raw1394_read (handle, raw1394_get_irm_id (handle),
+                CSR_REGISTER_BASE + CSR_BANDWIDTH_AVAILABLE,
+                sizeof (quadlet_t), &buffer);
+        if (result < 0)
+                return -1;
+  
+        buffer = ntohl (buffer);
+        compare = buffer;
+  
+        while (retry > 0) {
+                if (mode == RAW1394_MODIFY_ALLOC ) {
+                        if (compare < bandwidth) {
+                                return -1;
+                        }
+  
+                        swap = compare - bandwidth;
+                }
+                else {
+                        swap = compare + bandwidth;
+  
+                        if( swap > MAXIMUM_BANDWIDTH ) {
+                                swap = MAXIMUM_BANDWIDTH;
+                        }
+                }
+                
+                result = raw1394_lock (handle, raw1394_get_irm_id (handle),
+                                        CSR_REGISTER_BASE + CSR_BANDWIDTH_AVAILABLE, 
+                                        RAW1394_EXTCODE_COMPARE_SWAP, ntohl(swap), ntohl(compare),
+                                        &new);
+                if (result < 0)
+                        return -1;
+                
+                new = ntohl (new);
+                
+                if (new != compare) {
+                        compare = new;
+                        retry--;
+                        if ( retry == 0 )
+                                return -1;
+                }
+                else {
+                        /* Success. */
+                        retry = 0;
+                        return 0;
+                }
+        }
+  
+        return 0;
+}
+
+int ieee1394_channel_modify (raw1394handle_t handle, unsigned int channel,
+	enum raw1394_modify_mode mode)
+{
+        quadlet_t buffer;
+        int result;
+        nodeaddr_t addr = CSR_REGISTER_BASE;
+        unsigned int c = channel;
+        quadlet_t compare, swap = 0, new;
+        
+        if (c > 31 && c < 64) {
+                addr += CSR_CHANNELS_AVAILABLE_LO;
+                c -= 32;
+        } else if (c < 64)
+                addr += CSR_CHANNELS_AVAILABLE_HI;
+        else
+                return -1;
+        c = 31 - c;
+  
+        result = raw1394_read (handle, raw1394_get_irm_id (handle), addr, 
+                sizeof (quadlet_t), &buffer);
+        if (result < 0)
+                return -1;
+        
+        buffer = ntohl (buffer);
+  
+        if ( mode == RAW1394_MODIFY_ALLOC ) {
+                if( (buffer & (1 << c)) == 0 )
+                        return -1;
+                swap = htonl (buffer & ~(1 << c));
+        }
+        else if ( mode == RAW1394_MODIFY_FREE ) {
+                if ( (buffer & (1 << c)) != 0 )
+                        return -1;
+                swap = htonl (buffer | (1 << c));
+        }
+  
+        compare = htonl (buffer);
+  
+        result = raw1394_lock (handle, raw1394_get_irm_id (handle), addr,
+                                RAW1394_EXTCODE_COMPARE_SWAP, swap, compare, &new);
+        if ( (result < 0) || (new != compare) )
+                return -1;
+  
+        return 0;
+}
