@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1999-2004 Andreas Bombe, Dan Maas, Manfred Weihs, and
  *    Christian Toegel
+ * Copyright (C) 2008 Dan Dennedy <dan@dennedy.org>
  *
  * This library is licensed under the GNU Lesser General Public License (LGPL),
  * version 2.1 or later. See the file COPYING.LIB in the distribution for
@@ -106,6 +107,11 @@ enum raw1394_iso_disposition {
 
 	/* (transmission only) - like ISO_STOP, but don't wait for the buffer to empty */
 	RAW1394_ISO_STOP_NOSYNC = 4,
+	
+	/* (transmission only) - use this if the data you need for the packet was not ready yet
+	 * The packet queueing will retry at a later point
+	 */
+	RAW1394_ISO_AGAIN = 5,
 };
 
 enum raw1394_modify_mode {
@@ -113,7 +119,6 @@ enum raw1394_modify_mode {
 	RAW1394_MODIFY_FREE
 };
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -151,6 +156,7 @@ typedef enum raw1394_iso_disposition (*raw1394_iso_recv_handler_t)(
  * @irq_interval: maximum latency of wake-ups, in packets (-1 if you don't care)
  *
  * Allocates all user and kernel resources necessary for isochronous transmission.
+ * Channel and bandwidth allocation at the IRM is not performed.
  *
  * Returns: 0 on success or -1 on failure (sets errno)
  **/
@@ -322,6 +328,25 @@ void raw1394_iso_stop(raw1394handle_t handle);
  **/
 void raw1394_iso_shutdown(raw1394handle_t handle);
 
+/**
+ * raw1394_read_cycle_timer - get the current value of the cycle timer
+ * @handle: libraw1394 handle
+ * @cycle_timer: buffer for Isochronous Cycle Timer
+ * @local_time: buffer for local system time in microseconds since Epoch
+ *
+ * Simultaneously reads the cycle timer register together with the system clock.
+ *
+ * Format of @cycle_timer, from MSB to LSB: 7 bits cycleSeconds (seconds, or
+ * number of cycleCount rollovers), 13 bits cycleCount (isochronous cycles, or
+ * cycleOffset rollovers), 12 bits cycleOffset (24.576 MHz clock ticks, not
+ * provided on some hardware). The union of cycleSeconds and cycleCount is the
+ * current cycle number. The nominal duration of a cycle is 125 microseconds.
+ *
+ * Returns: the error code of the ioctl, or 0 if successful.
+ **/
+int raw1394_read_cycle_timer(raw1394handle_t handle,
+                             u_int32_t *cycle_timer, u_int64_t *local_time);
+
 typedef int raw1394_errcode_t;
 #define raw1394_make_errcode(ack, rcode) (((ack) << 16) | rcode)
 #define raw1394_internal_err(errcode) ((errcode) < 0)
@@ -338,7 +363,7 @@ typedef int raw1394_errcode_t;
  * roughly the same meaning.
  *
  * Returns: the error code of the last raw1394_read(), raw1394_write(),
- * raw1394_lock() or raw1394_iso_write().  The error code is either an internal
+ * raw1394_lock().  The error code is either an internal
  * error (i.e. not a bus error) or a combination of acknowledge code and
  * response code, as appropriate.
  *
@@ -374,9 +399,14 @@ int raw1394_errcode_to_errno(raw1394_errcode_t errcode);
  * processes.  It is allowed to create and use multiple handles, however.  Use
  * one handle per thread which needs it in the multithreaded case.
  *
+ * The default device node is /dev/raw1394, but one can override the default
+ * by setting environment variable RAW1394DEV. However, if RAW1394DEV points to
+ * a non-existant or invalid device node, then it also attempts to open the
+ * default device node.
+ *
  * Returns: the created handle or %NULL when initialization fails. In the latter
- * case errno either contains some OS specific error code or %0 if the error is
- * that libraw1394 and raw1394 don't support each other's protocol versions.
+ * case errno either contains some OS specific error code or EPROTO if
+ * libraw1394 and raw1394 don't support each other's protocol versions.
  **/
 raw1394handle_t raw1394_new_handle(void);
 
@@ -400,6 +430,11 @@ void raw1394_destroy_handle(raw1394handle_t handle);
  * command-line programs that already know what port they want. If
  * raw1394_set_port() returns ESTALE, retries automatically.
  *
+ * The default device node is /dev/raw1394, but one can override the default
+ * by setting environment variable RAW1394DEV. However, if RAW1394DEV points to
+ * a non-existant or invalid device node, then it also attempts to open the
+ * default device node.
+ *
  * Returns: the new handle on success or NULL on failure
  **/
 raw1394handle_t raw1394_new_handle_on_port(int port);
@@ -423,7 +458,8 @@ int raw1394_busreset_notify (raw1394handle_t handle, int off_on_switch);
  * the blocking behaviour in raw1394_loop_iterate().  It must not be used for
  * anything else.
  *
- * Returns: the fd used for communication with the raw1394 kernel module.
+ * Returns: the fd used for communication with the raw1394 kernel module or -1
+ * on failure (sets errno).
  **/
 int raw1394_get_fd(raw1394handle_t handle);
 
@@ -568,7 +604,7 @@ typedef int (*bus_reset_handler_t)(raw1394handle_t, unsigned int generation);
  * Sets the handler to be called on every bus reset to @new_h.
  * The default handler just calls raw1394_update_generation().
  *
- * Returns: the old handler
+ * Returns: the old handler or NULL on failure (sets errno)
  **/
 bus_reset_handler_t raw1394_set_bus_reset_handler(raw1394handle_t handle,
                                                   bus_reset_handler_t new_h);
@@ -583,7 +619,8 @@ bus_reset_handler_t raw1394_set_bus_reset_handler(raw1394handle_t handle,
  * The generation number of the handle is not automatically updated,
  * raw1394_update_generation() has to be used for this.
  *
- * Returns: the generation number associated with the handle 
+ * Returns: the generation number associated with the handle or UINT_MAX on
+* failure.
  **/
 unsigned int raw1394_get_generation(raw1394handle_t handle);
 
@@ -616,10 +653,10 @@ typedef int (*tag_handler_t)(raw1394handle_t, unsigned long tag,
  *
  * Care must be taken when replacing the tag handler and calling the synchronous
  * versions of the transaction functions (i.e. raw1394_read(), raw1394_write(),
- * raw1394_lock(), raw1394_iso_write()) since these do pass pointers to &struct
+ * raw1394_lock()) since these do pass pointers to &struct
  * raw1394_reqhandle as the tag and expect the callback to be invoked.
  *
- * Returns: the old handler
+ * Returns: the old handler or NULL on failure (sets errno)
  **/
 tag_handler_t raw1394_set_tag_handler(raw1394handle_t handle,
                                       tag_handler_t new_h);
@@ -637,7 +674,7 @@ typedef int (*arm_tag_handler_t)(raw1394handle_t handle, unsigned long arm_tag,
  * arrived. The default action is to call the arm_callback in the 
  * raw1394_arm_reqhandle pointed to by arm_tag.
  *
- * Returns: old handler
+ * Returns: old handler or NULL on failure (sets errno)
  **/
 arm_tag_handler_t raw1394_set_arm_tag_handler(raw1394handle_t handle,
                                       arm_tag_handler_t new_h);
@@ -657,7 +694,7 @@ typedef int (*fcp_handler_t)(raw1394handle_t, nodeid_t nodeid, int response,
  * In order to actually get FCP events, you have to enable it with
  * raw1394_start_fcp_listen() and can stop it with raw1394_stop_fcp_listen().
  *
- * Returns: the old handler
+ * Returns: the old handler or NULL on failure (sets errno)
  **/
 fcp_handler_t raw1394_set_fcp_handler(raw1394handle_t handle, fcp_handler_t new_h);
 
@@ -824,9 +861,9 @@ int raw1394_start_phy_packet_write(raw1394handle_t handle,
 /**
  * raw1394_start_read - initiate a read transaction
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
- * @length: amount of data to read
+ * @length: amount of bytes of data to read
  * @buffer: pointer to buffer where data will be saved
  * @tag: data to identify the request to completion handler
  *
@@ -848,9 +885,9 @@ int raw1394_start_read(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_start_write - initiate a write transaction
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to write to
- * @length: amount of data to write
+ * @length: amount of bytes of data to write
  * @data: pointer to data to be sent
  * @tag: data to identify the request to completion handler
  *
@@ -872,7 +909,7 @@ int raw1394_start_write(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_start_lock - initiate a 32-bit compare-swap lock transaction
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
  * @extcode: extended transaction code determining the lock operation
  * @data: data part of lock parameters
@@ -897,7 +934,7 @@ int raw1394_start_lock(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_start_lock64 - initiate a 64-bit compare-swap lock transaction
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
  * @extcode: extended transaction code determining the lock operation
  * @data: data part of lock parameters
@@ -920,13 +957,13 @@ int raw1394_start_lock64(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
                        octlet_t *result, unsigned long tag);
 
 /**
- * raw1394_start_read - initiate asynchronous stream
+ * raw1394_start_async_stream - initiate asynchronous stream
  * @handle: libraw1394 handle
  * @channel: the isochronous channel number to send on
  * @tag: data to be put into packet's tag field
  * @sy: data to be put into packet's sy field
  * @speed: speed at which to send
- * @length: amount of data to send
+ * @length: amount of data to send, in bytes
  * @data: pointer to data to send
  * @rawtag: data to identify the request to completion handler
  *
@@ -944,8 +981,8 @@ int raw1394_start_async_stream(raw1394handle_t handle, unsigned int channel,
 /**
  * raw1394_start_async_send - send an asynchronous packet
  * @handle: libraw1394 handle
- * @length: the amount of quadlets of data to send
- * @header_length: the number of quadlets in the header
+ * @length: the amount of bytes of data to send
+ * @header_length: the number of bytes in the header
  * @expect_response: indicate with a 0 or 1 whether to receive a completion event
  * @data: pointer to data to send
  * @rawtag: data to identify the request to completion handler
@@ -972,11 +1009,13 @@ int raw1394_start_async_send(raw1394handle_t handle,
 /**
  * raw1394_read - send async read request to a node and wait for response.
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
- * @length: amount of data to read in quadlets
+ * @length: amount of bytes of data to read
  * @buffer: pointer to buffer where data will be saved
  *
+ * If @length is %4, a quadlet read request is used.
+ *
  * This does the complete transaction and will return when it's finished.  It
  * will call raw1394_loop_iterate() as often as necessary, return values of
  * handlers called will be therefore lost.
@@ -989,11 +1028,13 @@ int raw1394_read(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_write - send async write request to a node and wait for response.
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to write to
- * @length: amount of data to write in quadlets
+ * @length: amount of bytes of data to write
  * @data: pointer to data to be sent
  *
+ * If @length is %4, a quadlet write request is used.
+ *
  * This does the complete transaction and will return when it's finished.  It
  * will call raw1394_loop_iterate() as often as necessary, return values of
  * handlers called will be therefore lost.
@@ -1006,7 +1047,7 @@ int raw1394_write(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_lock - send 32-bit compare-swap lock request and wait for response.
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
  * @extcode: extended transaction code determining the lock operation
  * @data: data part of lock parameters
@@ -1026,7 +1067,7 @@ int raw1394_lock(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 /**
  * raw1394_lock64 - send 64-bit compare-swap lock request and wait for response.
  * @handle: libraw1394 handle
- * @node: target node
+ * @node: target node ID
  * @addr: address to read from
  * @extcode: extended transaction code determining the lock operation
  * @data: data part of lock parameters
@@ -1050,7 +1091,7 @@ int raw1394_lock64(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
  * @tag: data to be put into packet's tag field
  * @sy: data to be put into packet's sy field
  * @speed: speed at which to send
- * @length: amount of data to send
+ * @length: amount of bytes of data to send
  * @data: pointer to data to send
  *
  * Returns: 0 on success or -1 on failure (sets errno)
@@ -1062,8 +1103,8 @@ int raw1394_async_stream(raw1394handle_t handle, unsigned int channel,
 /**
  * raw1394_async_send
  * @handle: libraw1394 handle
- * @length: the amount of quadlets of data to send
- * @header_length: the number of quadlets in the header
+ * @length: the amount of bytes of data to send
+ * @header_length: the number of bytes in the header
  * @expect_response: indicate with a 0 or 1 whether to receive a completion event
  * @data: pointer to data to send
  *
@@ -1114,7 +1155,7 @@ const char *raw1394_get_libversion(void);
  * raw1394_update_config_rom - updates the configuration ROM of a host
  * @handle: libraw1394 handle
  * @new_rom: a pointer to the new ROM image
- * @size: the size of the new ROM image in quadlets
+ * @size: the size of the new ROM image in bytes
  * @rom_version: the version numer of the current version, not the new
  *
  * @rom_version must be the current
@@ -1177,95 +1218,6 @@ raw1394_channel_modify (raw1394handle_t handle, unsigned int channel,
 	enum raw1394_modify_mode mode);
 
 
-/**
- * iso_handler_t - DEPRECATED
- * @handle: libraw1394 handle
- *
- * DEPRECATED
- **/
-typedef int (*iso_handler_t)(raw1394handle_t, int channel, size_t length,
-                             quadlet_t *data);
- 
-/**
- * raw1394_set_iso_handler - set isochronous packet handler (DEPRECATED)
- * @handle: libraw1394 handle
- * @new_h: pointer to new handler
- *
- * DEPRECATED
- *
- * Sets the handler to be called when an isochronous packet is received to
- * @new_h and returns the old handler.  The default handler does nothing.
- *
- * In order to actually get iso packet events, receiving on a specific channel
- * first has to be enabled with raw1394_start_iso_rcv() and can be stopped again
- * with raw1394_stop_iso_rcv().
- **/
-iso_handler_t raw1394_set_iso_handler(raw1394handle_t handle,
-                                      unsigned int channel,
-                                      iso_handler_t new_h);
-/**
- * raw1394_start_iso_rcv - enable isochronous receiving (DEPRECATED)
- * @handle: libraw1394 handle
- * @channel: channel number to start receiving on
- *
- * DEPRECATED
- *
- * Enables the reception of isochronous packets in @channel on @handle.
- * Isochronous packets are then passed to the callback specified with
- * raw1394_set_iso_handler().
- **/
-int raw1394_start_iso_rcv(raw1394handle_t handle, unsigned int channel);
-
-/**
- * raw1394_stop_iso_rcv - stop isochronous receiving (DEPRECATED)
- * @handle: libraw1394 handle
- * @channel: channel to stop receiving on
- *
- * DEPRECATED
- *
- * Stops the reception of isochronous packets in @channel on @handle.
- **/
-int raw1394_stop_iso_rcv(raw1394handle_t handle, unsigned int channel);
-
-/**
- * raw1394_start_iso_write - initiate an isochronous packet write (DEPRECATED)
- * @handle: libraw1394 handle
- * @channel: channel number on which to send on
- * @tag: data to be put into packet's tag field
- * @sy: data to be put into packet's sy field
- * @speed: speed at which to send
- * @length: amount of data to send
- * @data: pointer to data to send
- * @rawtag: data to identify the request to completion handler
- *
- * DEPRECATED
- *
- * This function starts the specified isochronous packet transmission and
- * returns %0 for success and a negative number for an error, in which case
- * errno will be set.
- *
- * When the send completes, a raw1394_loop_iterate() will call the tag handler
- * and pass it the tag and error code of the transaction.  @tag should therefore
- * be set to something that uniquely identifies this transaction (e.g. a struct
- * pointer casted to unsigned long).
- *
- * Isochronous packets are automatically
- **/
-int raw1394_start_iso_write(raw1394handle_t handle, unsigned int channel,
-                            unsigned int tag, unsigned int sy,
-                            unsigned int speed, size_t length, quadlet_t *data,
-                            unsigned long rawtag);
-
-/**
- * raw1394_iso_write - DEPRECATED
- * @handle: libraw1394 handle
- *
- * DEPRECATED
- **/
-int raw1394_iso_write(raw1394handle_t handle, unsigned int channel,
-                      unsigned int tag, unsigned int sy, unsigned int speed,
-                      size_t length, quadlet_t *data);
-
 #ifdef __cplusplus
 }
 #endif
